DROP PROCEDURE GT_TranDispatcher;
CREATE PROCEDURE GT_TranDispatcher
(	IN  pref     VARCHAR  (32),
	IN  objType  NVARCHAR (20),
	IN  objKey   NVARCHAR (255),
	IN  action   NVARCHAR (1),
	OUT errCode  INT,
	OUT errText  NVARCHAR (200)
)
AS
/* ---------------------------------------------------------
This procedure is part of the GazIntech transaction-
notification dispatcher, which is licensed under the terms
below:

Copyright (c) 2017 GazIntech LLC
info@gazintech.ru
 www.gazintech.ru

Permission to use, copy, modify, and/or distribute this
software for any purpose with or without fee is hereby
granted, provided that the above copyright notice and this
permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
OF THIS SOFTWARE.
--------------------------------------------------------- */
	invPrefix    NVARCHAR (256) =
	'	DO BEGIN
		DECLARE MYCOND CONDITION;
		DECLARE error NVARCHAR(200);
	';
	-- We have found no better way of returning data from dynamic SQL than
	-- by throwing an exception:
	invPostfix   NVARCHAR(256) =
	'
		IF COALESCE( :error, '''' ) <> '''' THEN
			SIGNAL MYCOND
			SET MESSAGE_TEXT = ''GT_[[['' || :error;
		END IF;
		END;
	';
	procName     VARCHAR  (255 );
	params       NVARCHAR (255 );
	invocation   NVARCHAR (1024);	
	errFull      NVARCHAR (4096);
	errCodeS     VARCHAR  (50 );
	tokLoc       INT;
	maxLen       INT;

	CURSOR procs ( c_pref VARCHAR(32), c_schema VARCHAR (64 ) ) FOR
	SELECT NAME FROM SYS.P_PROCEDURES_
	WHERE
		SCHEMA = :c_schema AND 
		NAME LIKE :c_pref || '_%';
		
BEGIN SEQUENTIAL EXECUTION
	DECLARE EXIT HANDLER FOR SQLEXCEPTION
	BEGIN
		errCode := ::SQL_ERROR_CODE;
		errText := 'Error in GT_TranDispatcher: ' ||
			SUBSTRING(::SQL_ERROR_MESSAGE, 0, 171);
	END;
	
	params := '(' || '''' || :objType || ''', ' || '''' || :objKey ||
	          ''',' || '''' || :action || ''',' || 'error' || ')';
	         
	errCode := 0;
	errText := 'All is well.';
	OPEN procs( :pref, CURRENT_SCHEMA );
	WHILE 1 = 1 DO
		FETCH procs INTO procName;
		IF procs::NotFound THEN BREAK; END IF;
		invocation := 'CALL ' || '"' || :procName || '"' || :params || ';' ;
		invocation := invPrefix || invocation || invPostfix;
		BEGIN
			DECLARE EXIT HANDLER FOR SQLEXCEPTION
			BEGIN
				maxLen   := 200 - LENGTH( :procName ) - 3;			
				tokLoc := LOCATE( ::SQL_ERROR_MESSAGE, 'GT_[[[' );
				IF :tokLoc > 0 THEN
					errCode := 1;
					errFull := SUBSTRING( ::SQL_ERROR_MESSAGE, :tokLoc + 6 );					
					IF LENGTH( :errFull ) > :maxLen THEN
						errFull := SUBSTRING( :errFull, 0, :maxLen - 3 ) || '...';
					END IF;
					errText := :errFull || ' [' || :procName || ']';
				ELSE
					errCode := ::SQL_ERROR_CODE;
					errFull := ::SQL_ERROR_MESSAGE;
				END IF;				
				IF LENGTH( :errFull ) > :maxLen THEN
					errFull := SUBSTRING( :errFull, 0, :maxLen - 3 ) || '...';
				END IF;
				errText := :errFull || ' [' || :procName || ']';
			END;
			EXEC :invocation;
		END;
		
		IF :errCode <> 0 THEN BREAK; END IF;
	END WHILE;
	CLOSE procs;
END;
