IF Object_ID(N'GT_TranDispatcher') IS NOT NULL
	DROP PROCEDURE [dbo].GT_TranDispatcher
GO

CREATE PROCEDURE GT_TranDispatcher
(	@prefix  VARCHAR(32),
	@type    NVARCHAR(20), 				
	@key     NVARCHAR(255),
	@action  NVARCHAR(1),
	@errCode INT            OUTPUT,
	@errText NVARCHAR(200)  OUTPUT
)
AS
/* ---------------------------------------------------------
This procedure is part of the GazIntech transaction-
notification dispatcher, which is licensed under the terms
below:

Copyright (c) 2017 GazIntech LLC
info@gazintech.ru
 www.gazintech.ru

Permission to use, copy, modify, and/or distribute this
software for any purpose with or without fee is hereby
granted, provided that the above copyright notice and this
permission notice appear in all copies.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS
ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO
EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
OF THIS SOFTWARE.
--------------------------------------------------------- */

DECLARE @procName      AS VARCHAR (255)
DECLARE @paramTemplate AS VARCHAR (255)
DECLARE @params        AS NVARCHAR(255)
DECLARE @sourceProc    AS VARCHAR (255)
DECLARE @outParDef     AS NVARCHAR(255)
DECLARE @maxLen        AS INT

SET @sourceProc = ''
SET @params = '@type, @key, @action, @errText OUTPUT'
SET @outParDef =
'	@type    NVARCHAR(20), 				
	@key     NVARCHAR(255),
	@action  NVARCHAR(1),
	@errText NVARCHAR(200)  OUTPUT
'
DECLARE @invocation AS NVARCHAR(255)
DECLARE Procs CURSOR STATIC FOR
	SELECT name FROM sys.procedures
	WHERE
		[type] = 'P' AND
		name LIKE @prefix + '_%'
	ORDER BY name ASC;
	
SET @errCode = 0;
OPEN Procs

WHILE ( 1 = 1 )
BEGIN
	FETCH NEXT FROM Procs INTO @procName
	IF (@@FETCH_STATUS <> 0) BREAK
	SET @invocation = 'EXEC ' + @procName + ' ' + @params
	BEGIN TRY
		EXEC sp_executesql @invocation, @outParDef,
				 @type = @type, @key = @key, @action = @action,
				 @errText = @errText OUTPUT
	END TRY
	BEGIN CATCH
		IF @procName <> ERROR_PROCEDURE()
			SET @sourceProc = '->' + ERROR_PROCEDURE()
		SET @errCode = ERROR_NUMBER();
		SET @errText = @procName + @sourceProc + ': line ' + 
		CAST(ERROR_LINE() AS VARCHAR(8)) + ': ' + ERROR_MESSAGE()
		BREAK
	END CATCH

	IF COALESCE( @errText, '' ) <> '' SET @errCode = 1;

	IF @errCode <> 0
	BEGIN
		SET @maxLen = 200 - LEN( @procName ) - 3;
		IF LEN( @errText ) > @maxLen
			SET @errText = SUBSTRING(@errText, 0, @maxLen - 3 ) + '...'
		SET @errText = @errText + ' [' + @procName + ']'
		BREAK
	END
END

CLOSE Procs
DEALLOCATE Procs
